/*
#######################################

Copyright 2017 Micro-Photon-Devices s.r.l.

SOFTWARE PRODUCT: PDM-IR_SDK

Micro-Photon-Devices (MPD) expressly disclaims any warranty for the SOFTWARE PRODUCT.
The SOFTWARE PRODUCT is provided 'As Is' without any express or implied warranty of any kind,
including but not limited to any warranties of merchantability, noninfringement, or
fitness of a particular purpose. MPD does not warrant or assume responsibility for the
accuracy or completeness of any information, text, graphics, links or other items contained
within the SOFTWARE PRODUCT. MPD further expressly disclaims any warranty or representation
to Authorized Users or to any third party.
In no event shall MPD be liable for any damages (including, without limitation, lost profits,
business interruption, or lost information) rising out of 'Authorized Users' use of or inability
to use the SOFTWARE PRODUCT, even if MPD has been advised of the possibility of such damages.
In no event will MPD be liable for loss of data or for indirect, special, incidental,
consequential (including lost profit), or other damages based in contract, tort
or otherwise. MPD shall have no liability with respect to the content of the
SOFTWARE PRODUCT or any part thereof, including but not limited to errors or omissions contained
therein, libel, infringements of rights of publicity, privacy, trademark rights, business
interruption, personal injury, loss of privacy, moral rights or the disclosure of confidential
information.

#######################################
*/
#ifndef __PDM_IR_SDK_h__
#define __PDM_IR_SDK_h__


#pragma once

#define WIN32_LEAN_AND_MEAN		/**< Exclude rarely-used stuff from Windows headers */

/**
\mainpage PDM-IR Software Development Kit (PDM-IR_SDK).
The MPD PDM-IR is based on a InGaAs/InP SPAD for the detection of near-infrared single photons up to 1700 nm. 
The module includes a pulse generator for gating the detector, a front-end circuit for avalanche sensing and 
a fast circuitry for detector quenching and resetting. 
The internal counters monitor the major signals (photons, internal trigger, trigger in, Aux In, valid gate).

\b IMPORTANT
In order to execute a program which links to the SDK libraries, the following file is required:
\verbatim
PDM-IR_SDK.dll       Software development kit library
\endverbatim

*/

/**\file PDM-IR_SDK.h
PDM-IR software development kit.
This C header contains all the functions to operate the PDM-IR in user defined applications.
*/

#ifdef __cplusplus
extern "C" {
#endif

/**Export to DLL define.
*/
#ifndef DllSDKExport 
	#if defined(_WIN32)   
		#ifdef PDM_IR_DLL_EXPORTS
			#define DllSDKExport __declspec(dllexport) /**< Export to DLL define */
		#else
			#ifndef PDM_IR_STATIC_LIBRARY
				#define DllSDKExport __declspec(dllimport)  /**< Export to DLL define */
			#else
				#define DllSDKExport 
			#endif
		#endif
	#else
		#define DllSDKExport __attribute__ ((visibility("default")))
	#endif
#endif

/**
* \defgroup Enum PDM-IR_SDK custom Types
Custom types used by the SDK
*/
/*@{*/

	
typedef unsigned char UINT8; /**< 8 bit unsigned definition*/
typedef   signed char INT8; /**< 8 bit signed definition*/
	
typedef   signed short INT16; /**< 16 bit unsigned definition*/
typedef unsigned short UINT16; /**< 16 bit signed definition*/

typedef unsigned int UINT32; /**< 32 bit unsigned definition*/
typedef   signed int INT32; /**< 32 bit signed definition*/

/**Error table enum.
Error code returned by the PDM-IR functions.
*/
typedef enum {
	RESULT_OK,					/**< result as expected. No errors.*/
	RESULT_DATA_OUT_LIMIT_HIGH, /**< At least one of the input values are above the maximum value accepted.*/
	RESULT_DATA_OUT_LIMIT_LOW,	/**< At least one of the input values are below the minimum value accepted.*/
	RESULT_DATA_ERROR,			/**< Error parsing the values referenced.*/
	RESULT_ERROR_COMMUNICATION	/**< Error communicating with the module.*/
} RESULT;

/**Temperature enum.
Temperature of the SPAD.
*/
typedef enum {
	THIGH = 0,		/**< High temperature: the dark count rate is the highest than other temperature modes, but no particular caution is required for module thermal stability.*/
	TMEDIUM = 1,	/**< Medium temperature: the dark count rate is the one shown in the test report. In this case the module requires to be placed on a reasonable heat sink for uninterrupted operation. */
	TLOW = 2,		/**< Low temperature: the dark count rate is very low. In this case the module requires a very good heath sink for optimal thermal dissipation and uninterrupted operation. */
	TLOWEST = 3		/**< Lowest temperature: the dark count rate is the lowest as possible. In this case the module requires a very good heath sink for thermal dissipation and a fan, in order to work without errors. Also, a low ambient temperature (of about 20-22C) will be required. */
} TEMP_VALUES;

/**hold Off Type enum.
The enum select the PDM-IR hold off type. 
*/
typedef enum {
	HOLDOFFTYPE_EDGE = 0,	/**<When the hold off time is over, the gate is not applied until the next rising edge of the gate window.*/
	HOLDOFFTYPE_LEVEL = 1	/**<The Gate window is applied after the hold off time is over. This type is required in free running mode*/	
} HOLDOFFTYPE;

/**Gate Mode enum.
The enum select the PDM-IR internal or external trigger. The chose trigger is in logic function with the Aux In
@see TRFUNCT
*/
typedef enum {
	GATEMODE_INTERNAL = 0,	/**<The Gate trigger is the internal trigger.*/	
	GATEMODE_EXTERNAL = 1	/**<The Gate trigger is the external trigger.*/	
} GATEMODE;

/**Gate Shape enum.
The enum specifies if the PDM-IR has a fixed width gate or the same width of the trigger gate, or is in free runing mode.
*/
typedef enum {
	GATESHAPE_FIXEDGATE = 0,	 /**<The Gate has a fixed width determinated by the Ton*/	
	GATESHAPE_FREEGATE = 1,		 /**<The Gate has the same digital shape and duration of the trigger gate signal*/	
	GATESHAPE_FREERUNNING = 2,		 /**<The SPAD is always on when not in hold off.*/	
} GATESHAPE;

/**Edge Type enum.
it selects the Edge sensitivity of the input.
*/
typedef enum {
	TREDGE_LH = 0,		/**<Rising Edge Trigger*/
	TREDGE_HL = 1,		/**<Falling Edge Trigger*/
} TREDGE;

/**Function Type enum.
Function between the trigger (Internal Trigger or Trigger In chosen by GATEMODE) and the Aux In. 
@see GATEMODE
*/
typedef enum {
	In_AND_Aux = 0,			/**<Selected trigger AND Aux In */
	In_OR_Aux = 1,			/**<Selected trigger OR Aux In */
	In_XOR_Aux = 2,			/**<Selected trigger XOR Aux In */
	notIn_AND_Aux = 3,		/**<Inverted Selected trigger AND Aux In */
	notIn_OR_Aux = 4,		/**<Inverted Selected trigger OR Aux In */
	notIn_XOR_Aux = 5,		/**<Inverted Selected trigger XOR Aux In */
	In_AND_notAux = 6,		/**<Selected trigger AND Inverted Aux In */
	In_OR_notAux = 7,		/**<Selected trigger OR Inverted Aux In */
	In_XOR_notAux = 8,		/**<Selected trigger XOR Inverted Aux In */
	notIn_AND_notAux = 9,	/**<Inverted Selected trigger AND Inverted Aux In */
	notIn_OR_notAux = 10,	/**<Inverted Selected trigger OR Inverted Aux In */
	notIn_XOR_notAux = 11,	/**<Inverted Selected trigger XOR Inverted Aux In */
	In_NAND_Aux = 12,		/**<Selected trigger NAND Aux In */
	In_NOR_Aux = 13,		/**<Selected trigger NOR Aux In */
	In_XNOR_Aux = 14,		/**<Selected trigger XNOR Aux In */
	notIn_NAND_Aux = 15,	/**<Inverted Selected trigger NAND Aux In */
	notIn_NOR_Aux = 16,		/**<Inverted Selected trigger NOR Aux In */
	notIn_XNOR_Aux = 17,	/**<Inverted Selected trigger XNOR Aux In */
	In_NAND_notAux = 18,	/**<Selected trigger NAND Inverted Aux In */
	In_NOR_notAux = 19,		/**<Selected trigger NOR Inverted Aux In */
	In_XNOR_notAux = 20,	/**<Selected trigger XNOR Inverted Aux In */
	notIn_NAND_notAux = 21,	/**<Inverted Selected trigger NAND Inverted Aux In */
	notIn_NOR_notAux = 22,	/**<Inverted Selected trigger NOR Inverted Aux In */
	notIn_XNOR_notAux = 23,	/**<Inverted Selected trigger XNOR Inverted Aux In */
	only_notIN = 24,		/**<Inverted Selected trigger */
	only_IN = 25			/**<Selected trigger */
} TRFUNCT;

/**Output Type enum.
it specifies the signal out from TLL out or NIM out.
*/
typedef enum {
	OUTMODE_TRIGGERGATE = 0,	/**<The Output reproduces the trigger gate signal*/
	OUTMODE_PHOTONOUT = 1,	/**<The Output reproduces the photon out signal*/
	OUTMODE_HOLDOFF = 2,	/**<The Output reproduces the hold off signal*/
	OUTMODE_VALIDGATE = 3,	/**<The Output reproduces the valid gate signal*/
	OUTMODE_INTERNALTRIGGER = 4,	/**<The Output reproduces the internal trigger signal*/
	OUTMODE_MODULESTATUS = 5	/**<The Output is high in case of error of the module (electrical signal of the RED led)*/
} OUTMODE;

/**Counter Status enum.
It indicates the couter status.
*/
typedef enum {
	COUNTERSTATUS_OFF = 0,			/**<The Counter is disabled*/
	COUNTERSTATUS_ON = 1,			/**<The Counter is enabled only for one period*/
	COUNTERSTATUS_Continuous = 2	/**<The Counter is enabled*/
} COUNTERSTATUS;

/**SPAD Type enum.
The SPAD type used in the module.
*/
typedef enum {
	TYPE_1 = 0x1,	/**< useful parameter for device statistics. Type1.*/
	TYPE_2 = 0x2,	/**< useful parameter for device statistics. Type2.*/
	TYPE_3 = 0x3,	/**< useful parameter for device statistics. Type3.*/
	TYPE_4 = 0x4,	/**< useful parameter for device statistics. Type4.*/
	TYPE_5 = 0x5,	/**< useful parameter for device statistics. Type5.*/
	TYPE_6 = 0x6,	/**< useful parameter for device statistics. Type6.*/
	TYPE_7 = 0x7,	/**< useful parameter for device statistics. Type7.*/
} SPAD_TYPE;

/**Parameter Status enum.
The enum indicates the actual status of a parameter.
*/
typedef enum {
	STATUSBITS_OFF = 0,				/**<The Parameter is disabled */
	STATUSBITS_ON = 1,				/**<The Parameter is enabled */
	STATUSBITS_Updating = 2,		/**<The Parameter is updating the state */
	STATUSBITS_SafeValue = 3,		/**<The Parameter has a safe value, used before update the value waiting the right update order */
	STATUSBITS_ToBeUpdated = 4,		/**<The Parameter needs to be updated */
	STATUSBITS_Updated = 5			/**<The Parameter is updated */
}STATUSBITS;


/**Warming Up module enum.
The enum indicates if the warm up of the module is completed or not.
*/
typedef enum {
	STATUSWARMUP_isWarmingUp = 0,	/**<The module is warming up*/
	STATUSWARMUP_isSteadyState = 1  /**<The module has a stable temperature*/
}STATUSWARMUP;

/**Module Configuration Structure.
It contains the configuration of the PDM-IR.
*/
typedef struct  {
	UINT16 Amplitude;								/**<The Gate Amplitude (Excess Bias) expressed in mV*/
	TEMP_VALUES Temperature;						/**<The temperature value */
	UINT32 HoldOff;									/**<The Hold Off value after an avalanche expressed in ns*/
	HOLDOFFTYPE HoldOffLevel;						/**<The Hold Off type: edge or level*/
	GATEMODE GateMode;								/**<The Gate Mode: trigger internal or external*/
	GATESHAPE GateShape;							/**<The Gate Shape: fixed gate, free gate or free running*/
	TREDGE TriggerEdge;							    /**<The Rising or falling edge of the trigger*/
	INT16 TriggerInTh;								/**<The Threshold of the trigger In expressed in mV*/
	INT16 AuxInTh;									/**<The Threshold of the Aux In expressed in mV*/
	TRFUNCT TriggerFunction;						/**<The Function between trigger (internal or external) and Aux In*/
	UINT32 Frequency;								/**<The Internal frequency of the PDM-IR in Hz*/
	UINT32 Ton;										/**<The On time of the gate in case of fixed gate width in ns*/
	OUTMODE TtlOut; 								/**<The output signal of the TTL out*/
	OUTMODE NimOut; 								/**<The output signal of the NIM out*/
	UINT8 DelayTriggerIn;							/**<The delay time of the Trigger In Signal*/
	UINT8 DelayTriggerInternal;						/**<The delay time of the Internal Trigger Signal*/
	UINT8 DelayAuxIn;								/**<The delay time of the Aux In Signal*/
	UINT8 DelayGate;								/**<The delay time of the Gate Signal*/
	UINT8 DelayNimOut;								/**<The delay time of the NIM out Signal*/
	UINT8 DelayTtlOut;								/**<The delay time of the TTL out Signal*/
	UINT16 CounterIntegrationTime;					/**<The counters' integration time*/
	COUNTERSTATUS CounterAvalancheStatus;			/**<The photon out counter status*/
	COUNTERSTATUS CounterValidGateStatus;			/**<The valid gate counter status*/
	COUNTERSTATUS CounterAuxInStatus;				/**<The Aux In counter status*/
	COUNTERSTATUS CounterTriggerInStatus;			/**<The trigger in counter status*/
	COUNTERSTATUS CounterInternalTriggerStatus;		/**<The internal trigger counter status*/
}MODULECONFIG;

/**Structure containing the information of the PDM-IR.*/
typedef struct {
	UINT16 SN;					/**<The PDM-IR Serial Number*/
	SPAD_TYPE SpadType;			/**<The Spad type*/
	char SpadName[16];			/**<The name of the Spad*/
	char FpgaFwVer[16];			/**<The Fpga Firmware Board*/
	char FpgaHwVer[16];			/**<The Fpga Hardware Board*/
	char McuFwVer[16];			/**<The Mcu Firmware Board*/
	char McuHwVer[16];			/**<The Mcu Hardware Board*/
}MODULEINFO;

/*@}*/

//------------ Constructor -----------------------------------------
/**
* \defgroup ConDes Constructror, destructor, error handling
Functions to Search, Open and Close the communication with the PDM-IR
*/
/*@{*/
/**Modules list Constructor.
It allocates a memory block to contain the list of the connected and recognized PDM-IR. It scans all the available USB port on the PC and when a PDM-IR is found,
it add the device to fill the list.
\param arrayDataSN Pointer to PDM-IR list handle. This parameter is referenced. 
\param numDevsFound Pointer to number of devices found. This parameter is referenced. 
\return RESULT_OK The list was successfully created and populated.
\return RESULT_ERROR_COMMUNICATION Error checking the device list.
\return RESULT_DATA_OUT_LIMIT_LOW no devices found.
*/
DllSDKExport RESULT PDMIR_SearchDevices(UINT16 *arrayDataSN, UINT16 *numDevsFound);

/**Open the communication with a module.
It opens the communication and allocates a memory block to manage the connected device, identified by the serial number.
\param serialNumber PDM-IR serial number
\return RESULT_OK The communication between the module and the pc is opened.
\return RESULT_ERROR_COMMUNICATION Error opening the communication channel.
*/
DllSDKExport RESULT PDMIR_OpenCommunication(UINT16 serialNumber);

/**Close the communication with a module.
It closes the communication and deallocates the memory. 
The module is identified by the serial number.
\param serialNumber PDM-IR serial number
\return RESULT_OK The communication between the module and the pc is correctly closed.
\return RESULT_ERROR_COMMUNICATION Error closing the communication channel.
*/
DllSDKExport RESULT PDMIR_CloseCommunication(UINT16 serialNumber);
/*@}*/
//------------ Set methods -----------------------------------------
/**
* \defgroup Set Set methods
Functions to set parameters of the PDM-IR
*/
/*@{*/
/**Set the gate amplitude of the PDM-IR.
\param serialNumber PDM-IR serial number
\param AmplimV Desired Amplitude in mV. Allowed range: 2000 mV to 7000 mV in fixed gate mode,2000 mV to 5000 mV in free gate mode and free running, at 100 mV steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetGateShape()
@see PDMIR_GetAmplitude()
*/
DllSDKExport RESULT PDMIR_SetAmplitude(UINT16 serialNumber, UINT16 AmplimV);

/**Set the working temperature of the PDM-IR.
\param serialNumber PDM-IR serial number
\param temp Desired Temperature as TEMP_VALUES.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetTemperature()
*/
DllSDKExport RESULT PDMIR_SetTemperature(UINT16 serialNumber, TEMP_VALUES temp);

/**Set the hold off time of the SPAD.
\param serialNumber PDM-IR serial number
\param holdOff Desired hold off time  in ns. Allowed range: 1000 ns to 3000000 ns, in 100 ns steps.
\param type Select edge or level hold off type.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetHoldOff()
*/
DllSDKExport RESULT PDMIR_SetHoldOff(UINT16 serialNumber, UINT32 holdOff, HOLDOFFTYPE type);

/**Set the gate mode of the PDM-IR.
\param serialNumber PDM-IR serial number
\param gm Desired gate mode as GATEMODE typedef. It can select internal trigger or trigger in.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
\return RESULT_DATA_ERROR The type is not HOLDOFFTYPE.
@see PDMIR_GetGateMode()
*/
DllSDKExport RESULT PDMIR_SetGateMode(UINT16 serialNumber, GATEMODE gm);

/**Set the gate shape of the PDM-IR.
\param serialNumber PDM-IR serial number
\param gs Desired gate shape as GATESHAPE typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetGateShape()
*/
DllSDKExport RESULT PDMIR_SetGateShape(UINT16 serialNumber, GATESHAPE gs);

/**Set the Trigger Edge of the PDM-IR.
\param serialNumber PDM-IR serial number
\param TriggerEdge trigger edge as TREDGE typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetTriggerEdge()
*/
DllSDKExport RESULT PDMIR_SetTriggerEdge(UINT16 serialNumber, TREDGE TriggerEdge);

/**Set the Trigger In Threshold of the PDM-IR.
\param serialNumber PDM-IR serial number
\param TriggerTh trigger In Threshold in mV. Allowed range: -2000 mV to 2000 mV, in 10 mV steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_SetTriggerInTh()
*/
DllSDKExport RESULT PDMIR_SetTriggerInTh(UINT16 serialNumber, INT16 TriggerTh);

/**Set the Aux In Threshold of the PDM-IR.
\param serialNumber PDM-IR serial number
\param TriggerTh Aux In Threshold in mV.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR. Allowed range: -2000 mV to 2000 mV, at 10 mV steps.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetAuxInTh()
*/
DllSDKExport RESULT PDMIR_SetAuxInTh(UINT16 serialNumber, INT16 TriggerTh);

/**Set the Function between Aux In and trigger (in or internal) of the PDM-IR.
\param serialNumber PDM-IR serial number
\param TriggerFunction trigger function as TRFUNCT typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetTriggerFunction()
*/
DllSDKExport RESULT PDMIR_SetTriggerFunction(UINT16 serialNumber, TRFUNCT TriggerFunction);

/**Set the internal trigger frequency in Hz of the PDM-IR.
\param serialNumber PDM-IR serial number
\param frequency Internal trigger frequency in Hz.  Allowed range: 100 Hz to 100000000 Hz (100 MHz), at 100 Hz steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetFrequency()
*/
DllSDKExport RESULT PDMIR_SetFrequency(UINT16 serialNumber, UINT32 frequency);

/**Set the Ton time in ns of the PDM-IR.
\param serialNumber PDM-IR serial number
\param tOn gate window time in ns.  Allowed range: 1 ns to 10000000 ns, at 1 ns steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetTon()
*/
DllSDKExport RESULT PDMIR_SetTon(UINT16 serialNumber, UINT32 tOn);

/**Set the TTL ouput signal of the PDM-IR.
\param serialNumber PDM-IR serial number
\param TTLOut output signal as OUTMODE typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetTTLOut()
*/
DllSDKExport RESULT PDMIR_SetTTLOut(UINT16 serialNumber, OUTMODE TTLOut);

/**Set the NIM ouput signal of the PDM-IR.
\param serialNumber PDM-IR serial number
\param NIMOut output signal as OUTMODE typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetNIMOut()
*/
DllSDKExport RESULT PDMIR_SetNIMOut(UINT16 serialNumber, OUTMODE NIMOut);

/**Set the Trigger In delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Delay value in ns. Allowed range: 1 ns to 100 ns, at 1 ns steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetDelayTriggerIn()
*/
DllSDKExport RESULT PDMIR_SetDelayTriggerIn(UINT16 serialNumber, UINT8 delay);

/**Set the Internal Trigger delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Delay value in ns. Allowed range: 1 ns to 100 ns, at 1 ns steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetDelayTriggerInternal()
*/
DllSDKExport RESULT PDMIR_SetDelayTriggerInternal(UINT16 serialNumber, UINT8 delay);

/**Set the Aux In delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Delay value in ns. Allowed range: 1 ns to 100 ns, at 1 ns steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetDelayAuxIn()
*/
DllSDKExport RESULT PDMIR_SetDelayAuxIn(UINT16 serialNumber, UINT8 delay);

/**Set the Gate delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Delay value in ns. Allowed range: 1 ns to 100 ns, at 1 ns steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetDelayGate()
*/
DllSDKExport RESULT PDMIR_SetDelayGate(UINT16 serialNumber, UINT8 delay);

/**Set the NIM out delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Delay value in ns. Allowed range: 1 ns to 100 ns, at 1 ns steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetDelayNIMOut()
*/
DllSDKExport RESULT PDMIR_SetDelayNIMOut(UINT16 serialNumber, UINT8 delay);

/**Set the TTL out delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Delay value in ns. Allowed range: 1 ns to 100 ns, at 1 ns steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetDelayTTLOut()
*/
DllSDKExport RESULT PDMIR_SetDelayTTLOut(UINT16 serialNumber, UINT8 delay);

/**Set the counters integration time of the PDM-IR.
\param serialNumber PDM-IR serial number
\param integrationTime Integration time in ms.  Allowed range: 100 ms to 60000 ms, at 20 ms steps.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetCounterIntegrationTime()
*/
DllSDKExport RESULT PDMIR_SetCounterIntegrationTime(UINT16 serialNumber, UINT16 integrationTime);

/**Set the photon out counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param Cstatus counter status as COUNTERSTATUS typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetCounterAvalancheStatus()
*/
DllSDKExport RESULT PDMIR_SetCounterAvalancheEnable(UINT16 serialNumber, COUNTERSTATUS Cstatus);

/**Set the valid gate counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param Cstatus counter status as COUNTERSTATUS typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetCounterValidGateStatus()
*/
DllSDKExport RESULT PDMIR_SetCounterValidGateEnable(UINT16 serialNumber, COUNTERSTATUS Cstatus);

/**Set the Aux In counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param Cstatus counter status as COUNTERSTATUS typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetCounterAuxInStatus()
*/
DllSDKExport RESULT PDMIR_SetCounterAuxInEnable(UINT16 serialNumber, COUNTERSTATUS Cstatus);

/**Set the trigger In counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param Cstatus counter status as COUNTERSTATUS typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetCounterTriggerInStatus()
*/
DllSDKExport RESULT PDMIR_SetCounterTriggerInEnable(UINT16 serialNumber, COUNTERSTATUS Cstatus);

/**Set the internal trigger counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param Cstatus counter status as COUNTERSTATUS typedef.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
@see PDMIR_GetCounterInternalTriggerStatus()
*/
DllSDKExport RESULT PDMIR_SetCounterInternalTriggerEnable(UINT16 serialNumber, COUNTERSTATUS Cstatus);

/**Set the module status of the PDM-IR.
\param serialNumber PDM-IR serial number
\param Mstatus counter status as STATUSBITS typedef. Allowed values: STATUSBITS_OFF, STATUSBITS_ON
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The value set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The value set is lower than min value.
*/
DllSDKExport RESULT PDMIR_SetModuleOnOff(UINT16 serialNumber, STATUSBITS Mstatus);
/*@}*/
//------------ Get methods -----------------------------------------
/**
* \defgroup Get Get methods
Functions to get parameters of the PDM-IR
*/
/*@{*/
/**Get the gate amplitude of the PDM-IR.
\param serialNumber PDM-IR serial number
\param ampi Pointer to actual Amplitude in mV. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetGateShape()
@see PDMIR_SetAmplitude()
*/
DllSDKExport RESULT PDMIR_GetAmplitude(UINT16 serialNumber, UINT16 *ampi);

/**Get the working temperature of the PDM-IR.
\param serialNumber PDM-IR serial number
\param temp Pointer to actual Temperature as TEMP_VALUES. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetTemperature()
*/
DllSDKExport RESULT PDMIR_GetTemperature(UINT16 serialNumber, TEMP_VALUES *temp);

/**Get the hold off time of the PDM-IR.
\param serialNumber PDM-IR serial number
\param holdOff Pointer to actual Hold Off time in ns. This parameter is referenced.
\param type Pointer to the Hold Off type. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetHoldOff()
*/
DllSDKExport RESULT PDMIR_GetHoldOff(UINT16 serialNumber, UINT32 *holdOff, HOLDOFFTYPE *type);

/**Get the gate mode of the PDM-IR.
\param serialNumber PDM-IR serial number
\param gate Pointer to gate mode as GATEMODE typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetGateMode()
*/
DllSDKExport RESULT PDMIR_GetGateMode(UINT16 serialNumber, GATEMODE *gate);

/**Get the gate shape of the PDM-IR.
\param serialNumber PDM-IR serial number
\param shape Pointer to gate shape as GATESHAPE typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetGateShape()
*/
DllSDKExport RESULT PDMIR_GetGateShape(UINT16 serialNumber, GATESHAPE *shape);

/**Get the Trigger Edge of the PDM-IR.
\param serialNumber PDM-IR serial number
\param edge Pointer to the trigger edge as TREDGE typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetTriggerEdge()
*/
DllSDKExport RESULT PDMIR_GetTriggerEdge(UINT16 serialNumber, TREDGE *edge);

/**Get the Trigger In Threshold of the PDM-IR.
\param serialNumber PDM-IR serial number
\param TriggerInTh Pointer to the trigger In Threshold in mV. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetTriggerInTh()
*/
DllSDKExport RESULT PDMIR_GetTriggerInTh(UINT16 serialNumber, INT16 *TriggerInTh);

/**Get the Aux In Threshold of the PDM-IR.
\param serialNumber PDM-IR serial number
\param AuxInTh Pointer to the Aux In Threshold in mV. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetAuxInTh()
*/
DllSDKExport RESULT PDMIR_GetAuxInTh(UINT16 serialNumber, INT16 *AuxInTh);

/**Get the Function between Aux In and trigger (in or internal) of the PDM-IR.
\param serialNumber PDM-IR serial number
\param function Pointer to trigger function as TRFUNCT typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetTriggerFunction()
*/
DllSDKExport RESULT PDMIR_GetTriggerFunction(UINT16 serialNumber, TRFUNCT *function);

/**Get the internal trigger frequency in Hz of the PDM-IR.
\param serialNumber PDM-IR serial number
\param freq Pointer to Internal trigger frequency in Hz. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetFrequency()
*/
DllSDKExport RESULT PDMIR_GetFrequency(UINT16 serialNumber, UINT32 *freq);

/**Get the Ton time in ns of the PDM-IR.
\param serialNumber PDM-IR serial number
\param Ton Pointer to Ton time in ns. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetTon()
*/
DllSDKExport RESULT PDMIR_GetTon(UINT16 serialNumber, UINT32 *Ton);

/**Get the TTL ouput signal of the PDM-IR.
\param serialNumber PDM-IR serial number
\param TTLOut Pointer to output signal as OUTMODE typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetTTLOut()
*/
DllSDKExport RESULT PDMIR_GetTTLOut(UINT16 serialNumber, OUTMODE *TTLOut);

/**Get the NIM ouput signal of the PDM-IR.
\param serialNumber PDM-IR serial number
\param NIMOut Pointer to output signal as OUTMODE typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetNIMOut()
*/
DllSDKExport RESULT PDMIR_GetNIMOut(UINT16 serialNumber, OUTMODE *NIMOut);

/**Get the Trigger In delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Pointer to added delay at the signal in ns. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetDelayTriggerIn()
*/
DllSDKExport RESULT PDMIR_GetDelayTriggerIn(UINT16 serialNumber, UINT8 *delay);

/**Get the Internal Trigger delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Pointer to added delay at the signal in ns. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetDelayTriggerInternal()
*/
DllSDKExport RESULT PDMIR_GetDelayTriggerInternal(UINT16 serialNumber, UINT8 *delay);

/**Get the Aux In delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Pointer to added delay at the signal in ns. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetDelayAuxIn()
*/
DllSDKExport RESULT PDMIR_GetDelayAuxIn(UINT16 serialNumber, UINT8 *delay);

/**Get the Gate delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Pointer to added delay at the signal in ns. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetDelayGate()
*/
DllSDKExport RESULT PDMIR_GetDelayGate(UINT16 serialNumber, UINT8 *delay);

/**Get the NIM out delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Pointer to added delay at the signal in ns. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetDelayNIMOut()
*/
DllSDKExport RESULT PDMIR_GetDelayNIMOut(UINT16 serialNumber, UINT8 *delay);

/**Get the TTL out delay of the PDM-IR.
\param serialNumber PDM-IR serial number
\param delay Pointer to added delay at the signal in ns. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetDelayTTLOut()
*/
DllSDKExport RESULT PDMIR_GetDelayTTLOut(UINT16 serialNumber, UINT8 *delay);

/**Get the counters integration time of the PDM-IR.
\param serialNumber PDM-IR serial number
\param integrationTime Pointer to the integration time in ms. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetCounterIntegrationTime()
*/
DllSDKExport RESULT PDMIR_GetCounterIntegrationTime(UINT16 serialNumber, UINT16 *integrationTime);

/**Get the photon out counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param status Pointer to the counter status as COUNTERSTATUS typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetCounterAvalancheEnable()
*/
DllSDKExport RESULT PDMIR_GetCounterAvalancheStatus(UINT16 serialNumber, COUNTERSTATUS *status);

/**Get the valid gate counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param status Pointer to the counter status as COUNTERSTATUS typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetCounterValidGateEnable()
*/
DllSDKExport RESULT PDMIR_GetCounterValidGateStatus(UINT16 serialNumber, COUNTERSTATUS *status);

/**Get the Aux In counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param status Pointer to the counter status as COUNTERSTATUS typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetCounterAuxInEnable()
*/
DllSDKExport RESULT PDMIR_GetCounterAuxInStatus(UINT16 serialNumber, COUNTERSTATUS *status);

/**Get the trigger In counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param status Pointer to the counter status as COUNTERSTATUS typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetCounterTriggerInEnable()
*/
DllSDKExport RESULT PDMIR_GetCounterTriggerInStatus(UINT16 serialNumber, COUNTERSTATUS *status);

/**Get the internal trigger counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param status Pointer to the counter status as COUNTERSTATUS typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_SetCounterInternalTriggerEnable()
*/
DllSDKExport RESULT PDMIR_GetCounterInternalTriggerStatus(UINT16 serialNumber, COUNTERSTATUS *status);

/**Get the status of all the counter of the PDM-IR.
\param serialNumber PDM-IR serial number
\param avalancheStatus Pointer to the avalanche counter status as COUNTERSTATUS typedef. This parameter is referenced.
\param validGateStatus Pointer to the valid gate counter status as COUNTERSTATUS typedef. This parameter is referenced.
\param auxInStatus Pointer to the aux in counter status as COUNTERSTATUS typedef. This parameter is referenced.
\param triggerInStatus Pointer to the trigger in counter status as COUNTERSTATUS typedef. This parameter is referenced.
\param internalTriggerStatus Pointer to the internal trigger counter status as COUNTERSTATUS typedef. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
*/
DllSDKExport RESULT PDMIR_GetAllCountersStatus(UINT16 serialNumber, COUNTERSTATUS *avalancheStatus, COUNTERSTATUS *validGateStatus, COUNTERSTATUS *auxInStatus, COUNTERSTATUS *triggerInStatus, COUNTERSTATUS *internalTriggerStatus);

/**Get the photon out counter value of the PDM-IR.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated in the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterAvalancheValueAtIndex()
*/
DllSDKExport RESULT PDMIR_GetCounterAvalancheLastValue(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the valid gate counter value of the PDM-IR.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated in the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterValidGateValueAtIndex()
*/
DllSDKExport RESULT PDMIR_GetCounterValidGateLastValue(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the aux in counter value of the PDM-IR.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated in the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterAuxInValueAtIndex()
*/
DllSDKExport RESULT PDMIR_GetCounterAuxInLastValue(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the trigger in counter value of the PDM-IR.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated in the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterTriggerInValueAtIndex()
*/
DllSDKExport RESULT PDMIR_GetCounterTriggerInLastValue(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the internal trigger counter value of the PDM-IR.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated in the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterInternalTriggerValueAtIndex()
*/
DllSDKExport RESULT PDMIR_GetCounterInternalTriggerLastValue(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the value of all the counters of the PDM-IR.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param avalancheCounts Pointer to the counts value accumulated in the integration time in the avalanche counter. This parameter is referenced.
\param validGateCounts Pointer to the counts value accumulated in the integration time in the valid gate counter. This parameter is referenced.
\param auxInCounts Pointer to the counts value accumulated in the integration time in the aux in counter. This parameter is referenced.
\param triggerInCounts Pointer to the counts value accumulated in the integration time in the trigger in counter. This parameter is referenced.
\param internalTriggerCounts Pointer to the counts value accumulated in the integration time in the internal trigger counter. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetAllCountersValueAtIndex()
*/
DllSDKExport RESULT PDMIR_GetAllCountersLastValue(UINT16 serialNumber, UINT8 *index, UINT32 *avalancheCounts, UINT32 *validGateCounts, UINT32 *auxInCounts, UINT32 *triggerInCounts, UINT32 *internalTriggerCounts);

/**Get the photon out counter value of the PDM-IR at specified index.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated int the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The index value is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The index value is lower than min value.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterAvalancheLastValue()
*/
DllSDKExport RESULT PDMIR_GetCounterAvalancheValueAtIndex(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the valid gate counter value of the PDM-IR at specified index.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated int the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The index value is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The index value is lower than min value.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterValidGateLastValue()
*/
DllSDKExport RESULT PDMIR_GetCounterValidGateValueAtIndex(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the Aux In counter value of the PDM-IR at specified index.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated int the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The index value is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The index value is lower than min value.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterAuxInLastValue()
*/
DllSDKExport RESULT PDMIR_GetCounterAuxInValueAtIndex(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the trigger in counter value of the PDM-IR at specified index.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated int the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The index value is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The index value is lower than min value.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterTriggerInLastValue()
*/
DllSDKExport RESULT PDMIR_GetCounterTriggerInValueAtIndex(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the internal trigger counter value of the PDM-IR at specified index.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param counts Pointer to the counts value accumulated int the integration time. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The index value is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The index value is lower than min value.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetCounterInternalTriggerLastValue()
*/
DllSDKExport RESULT PDMIR_GetCounterInternalTriggerValueAtIndex(UINT16 serialNumber, UINT8 *index, UINT32 *counts);

/**Get the value of all the counters of the PDM-IR at specified index.
\param serialNumber PDM-IR serial number
\param index Pointer to the index of the rotating array. Index is a value between 0 and 19. This parameter is referenced.
\param avalancheCounts Pointer to the counts value accumulated in the integration time in the avalanche counter. This parameter is referenced.
\param validGateCounts Pointer to the counts value accumulated in the integration time in the valid gate counter. This parameter is referenced.
\param auxInCounts Pointer to the counts value accumulated in the integration time in the aux in counter. This parameter is referenced.
\param triggerInCounts Pointer to the counts value accumulated in the integration time in the trigger in counter. This parameter is referenced.
\param internalTriggerCounts Pointer to the counts value accumulated in the integration time in the internal trigger counter. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The index value is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The index value is lower than min value.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_GetAllCountersLastValue()
*/
DllSDKExport RESULT PDMIR_GetAllCountersValueAtIndex(UINT16 serialNumber, UINT8 *index, UINT32 *avalancheCounts, UINT32 *validGateCounts, UINT32 *auxInCounts, UINT32 *triggerInCounts, UINT32 *internalTriggerCounts);

/**Get the module information of the PDM-IR.
\param serialNumber PDM-IR serial number
\param info Pointer to the structure MODULEINFO. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
*/
DllSDKExport RESULT PDMIR_GetModuleInfo(UINT16 serialNumber, MODULEINFO *info);

/**Get the module status of the PDM-IR.
\param serialNumber PDM-IR serial number
\param mStatus Pointer to the module status. Allowed values: STATUSBITS_OFF, STATUSBITS_ON. This parameter is referenced.
\param mTemperature Pointer to the temperature status. Allowed values: STATUSBITS_OFF, STATUSBITS_ON, STATUSBITS_Updating. This parameter is referenced.
\param mSpad Pointer to the SPAD polarization status. Allowed values: STATUSBITS_OFF, STATUSBITS_ON, STATUSBITS_Updating. This parameter is referenced.
\param mGate Pointer to the gate status. Allowed values: STATUSBITS_OFF, STATUSBITS_ON, STATUSBITS_Updating. This parameter is referenced.
\param mWarm Pointer to the module temperature. This parameter is referenced.
\param errors Pointer to error code. Use PDMIR_ErrorTranslator() to convert code value to string. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_ERROR The received value has wrong format.
@see PDMIR_ErrorTranslator()
*/
DllSDKExport RESULT PDMIR_GetModuleStatus(UINT16 serialNumber, STATUSBITS *mStatus, STATUSBITS *mTemperature, STATUSBITS *mSpad, STATUSBITS *mGate, STATUSWARMUP *mWarm, UINT32 *errors);

/**Get the string description of an error code of the PDM-IR.
\param error error code.
\param stringOut Pointer to output char array. Array is at least 450 char lenght. This parameter is referenced.
\return null
@see PDMIR_GetModuleStatus()
*/
DllSDKExport void PDMIR_ErrorTranslator(UINT32 error, char *stringOut); 
/*@}*/
//------------ Configuration methods -----------------------------------------
/**
* \defgroup Configuration Configuration methods
Functions to manage the configurations of the PDM-IR
*/
/*@{*/

/**Store current configuration of the PDM-IR in the configNumber position.
\param serialNumber PDM-IR serial number
\param configNumber position where the configuration will be saved. Allowed values: from 4 to 10. If the position cointains a valid configuration doesn't overwrite the configuration
\param configName Pointer to a char array. Max length is 16 chars. The name needs to be terminated with 0. 
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR or there's a valid configuration in the configuration position.
\return RESULT_DATA_OUT_LIMIT_HIGH The position set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The position set is lower than min value.
@see PDMIR_LoadConfig()
*/
DllSDKExport RESULT PDMIR_SaveCurrentConfig(UINT16 serialNumber, UINT8 configNumber, char *configName);

/**Store current configuration of the PDM-IR in the configNumber position overwriting the existing one.
\param serialNumber PDM-IR serial number
\param configNumber position where save the configuration. Allowed values: from 4 to 10. If the position cointains a valid configuration, it's overwrote
\param configName Pointer to a char array. Max length is 16 chars. The name needs to be terminated with 0.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The position set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The position set is lower than min value.
@see PDMIR_LoadConfig()
*/
DllSDKExport RESULT PDMIR_SaveCurrentConfigForceOverWrite(UINT16 serialNumber, UINT8 configNumber, char *configName);

/**Delete the configuration stored in the configNumber position.
\param serialNumber PDM-IR serial number
\param configNumber position where the configuration will be deleted. Allowed values: from 4 to 10.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR or the configuration doesn't exist.
\return RESULT_DATA_OUT_LIMIT_HIGH The position set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The position set is lower than min value.
@see PDMIR_LoadConfig()
*/
DllSDKExport RESULT PDMIR_DeleteConfig(UINT16 serialNumber, UINT8 configNumber);

/**Set a stored configuration as power up configuration of the PDM-IR.
\param serialNumber PDM-IR serial number
\param configNumber Configuration position set as power up configuration. Allowed values: from 1 to 10.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR or the configuration in configNumber position doesn't exist.
\return RESULT_DATA_OUT_LIMIT_HIGH The position set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The position set is lower than min value.
@see PDMIR_LoadConfig()
*/
DllSDKExport RESULT PDMIR_SetConfigPowerUp(UINT16 serialNumber, UINT8 configNumber);

/**Load a stored configuration of the PDM-IR.
\param serialNumber PDM-IR serial number
\param configNumber position where the configuration is read and loaded. Allowed values: from 1 to 10.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR or the configuration doesn't exist.
\return RESULT_DATA_OUT_LIMIT_HIGH The position set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The position set is lower than min value.
@see PDMIR_SaveCurrentConfig()
@see PDMIR_SaveCurrentConfigForceOverWrite()
@see PDMIR_DeleteConfig()
@see PDMIR_SetConfigPowerUp()
*/
DllSDKExport RESULT PDMIR_LoadConfig(UINT16 serialNumber, UINT8 configNumber);

/**Get the current configuration parameters of the PDM-IR.
\param serialNumber PDM-IR serial number
\param config Pointer to MODULECONFIG structure.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
*/
DllSDKExport RESULT PDMIR_GetCurrentConfig(UINT16 serialNumber, MODULECONFIG *config);

/**Get a stored configuration of the PDM-IR.
\param serialNumber PDM-IR serial number
\param configNumber position where the configuration is read. Allowed values: from 1 to 10.
\param config Pointer to MODULECONFIG structure. This parameter is referenced.
\param configName Pointer to a char array cointaining the configuration name. Max length is 16 chars. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR.
\return RESULT_DATA_OUT_LIMIT_HIGH The position set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The position set is lower than min value.
*/
DllSDKExport RESULT PDMIR_GetConfigX(UINT16 serialNumber, UINT8 configNumber, MODULECONFIG *config, char *configName);

/**Get the stored configuration number set as power up configuration of the PDM-IR.
\param serialNumber PDM-IR serial number
\param configNumber Pointer to power up configuration position. Allowed values: from 1 to 10. This parameter is referenced.
\return RESULT_OK
\return RESULT_ERROR_COMMUNICATION Error communicating with the PDM-IR or the configuration doesn't exist.
\return RESULT_DATA_OUT_LIMIT_HIGH The position set is higher than max value.
\return RESULT_DATA_OUT_LIMIT_LOW The position set is lower than min value.
*/
DllSDKExport RESULT PDMIR_GetConfigPowerUp(UINT16 serialNumber, UINT8 *configNumber);

/*@}*/

/*@}*/

#ifdef __cplusplus
}
#endif

#endif //__PDM_IR_SDK_h__